/* 
 * Autor: Franz Brauße
 * License: GPL v2 
 */

package general;

import java.util.ArrayList;
import java.util.Timer;
import java.util.TimerTask;
import javax.swing.DefaultListModel;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

public class Log {
	
	public static final ImageIcon NETWORK_OFFLINE = new ImageIcon("ressources/network-offline.png");
	public static final ImageIcon NETWORK_IDLE = new ImageIcon("ressources/network-idle.png");
	public static final ImageIcon NETWORK_BUSY = new ImageIcon("ressources/network-transmit-receive.png");
	public static final ImageIcon NETWORK_ERROR = new ImageIcon("ressources/network-error.png");
	
	private final DefaultListModel jLog;
	private final JScrollPane jScrollPane1;
	private final JPanel jContentPane;
	private final Config cfg;
	private final JLabel networkState;
	
	private final ArrayList<String> open;
	
	public Log(Config cfg, JPanel jContentPane, JScrollPane jScrollPane1, DefaultListModel jLog, JLabel networkState) {
		this.cfg = cfg;
		this.jContentPane = jContentPane;
		this.jScrollPane1 = jScrollPane1;
		this.jLog = jLog;
		this.networkState = networkState;
		this.open = new ArrayList<String>();
		if (cfg.logShowAlways)
			this.show();
	}
	
	public void setNetworkState(ImageIcon icon) {
		this.networkState.setIcon(icon);
	}
	
	public void setNetworkBusy() {
		this.networkState.setIcon(NETWORK_BUSY);
	}
	public void setNetworkIdle() {
		this.networkState.setIcon(NETWORK_IDLE);
	}
	public void setNetworkError() {
		this.networkState.setIcon(NETWORK_ERROR);
	}
	public void setNetworkOffline() {
		this.networkState.setIcon(NETWORK_OFFLINE);
	}
	
	/**
	 * Adds a line to the main Log-JList.
	 * 
	 * @param entry the message to display
	 * @param initiator an arbitrary String uniquely identifying the calling method or procedure
	 * @param wait specifies whether the log-view shall be closed after this message.
	 * <p>If {@link Config#logHideDelay} is set and {@link Config#logShowAlways} is
	 * disabled - so to speak the log will be hidden after a certain delay automatically -, this
	 * value indicates whether the log shall be hidden after the delay.</p>
	 * <p>If some message(s) of the same <code>initiator</code> will definitely follow this one
	 * and belong to the same session / procedure, this value should be enabled. Otherwise - if
	 * this will/might be the last message of this <code>initiator</code> - <code>wait</code>
	 * should be set <code>false</code>.</p>
	 * <p>I.e. after "Connecting..." there should follow some message informing about the result,
	 * so <code>wait</code> should be set to <code>true</code>.</p>
	 */
	public void addEntry(final String message, final String initiator, final boolean wait) {
		this.jLog.addElement(message);
		if (this.open.size() == 0) {
			this.jScrollPane1.setVisible(true);
			this.jContentPane.updateUI();
		}
		
		final int r = this.open.indexOf(initiator);
		if (wait && r == -1)
			this.open.add(initiator);
		else if (!wait) {
			if (r != -1)
				this.open.remove(r);
			if (this.open.size() == 0 && !this.cfg.logShowAlways)
				closeAfterDelay();
		}
		this.jScrollPane1.updateUI();
	}
	
	private class lTim extends TimerTask {
		public void run() {
			if (Log.this.open.size() > 0) return;
			Log.this.jScrollPane1.setVisible(staysOpen());
			Log.this.jContentPane.updateUI();
		}
	}
	
	public void closeAfterDelay() {
		new Timer().schedule(new lTim(), this.cfg.logHideDelay);
	}
	
	public boolean staysOpen() {
		return (this.open.size() > 0);
	}
	
	public void clear() {
		this.jLog.clear();
	}
	
	public void hide() {
		this.jScrollPane1.setVisible(false);
		this.jContentPane.updateUI();
	}
	
	public void show() {
		this.jScrollPane1.setVisible(true);
		this.jContentPane.updateUI();
	}
}
